import {
  I18n, IDictionary, Router, Metadata, dayjs as moment, Loader,
} from '@zeedhi/core';
import { DialogService, Badge } from '@zeedhi/common';
import { executeExport } from '../server/server';
import { useMirage } from '../server/use-mirage';
import { RequestLog } from '../server/request-logger';

export class ExportacaoController {
  private utilities = Loader.getInstance('Utilities');

  private errorsController = Loader.getInstance('ErrorsController');

  public confirmExport() {
    const hasImportedData = localStorage.getItem('hasImportedData') || '';
    const item = localStorage.getItem('request-log') || '[]';
    const logs: IDictionary = JSON.parse(item);
    if (hasImportedData === 'true' && logs.length > 0) {
      DialogService.show({
        name: 'askForExport',
        type: 'warning',
        title: I18n.translate('scm::ATTENTION'),
        text: I18n.translate('scm::WANT_TO_EXPORT'),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: I18n.translate('scm::YES'),
            events: {
              click: () => {
                if (navigator.onLine) {
                  useMirage(false);
                  if (Router.getCurrentRoute().name !== '') {
                    Router.push('/');
                    this.utilities.validateActiveUser().then((valid: any) => {
                      if (valid) {
                        this.utilities.verifyVersionStatus().then((versionValid: any) => {
                          if (versionValid) {
                            this.executeExportacao();
                          }
                        });
                      }
                    });
                  } else {
                    this.utilities.validateActiveUser().then((valid: any) => {
                      if (valid) {
                        this.utilities.verifyVersionStatus().then((versionValid: any) => {
                          if (versionValid) {
                            this.executeExportacao();
                          }
                        });
                      }
                    });
                  }
                } else {
                  DialogService.show({
                    name: 'errorExport',
                    type: 'error',
                    title: I18n.translate('scm::ERROR'),
                    text: I18n.translate('scm::EXPORT_CONNECTION_FAIL'),
                    buttons: [
                      {
                        name: 'ok',
                        component: 'ZdButton',
                        label: I18n.translate('scm::OK'),
                        events: {
                          click: () => {
                            DialogService.hide();
                          },
                        },
                      },
                    ],
                  });
                }
              },
            },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: I18n.translate('scm::NO'),
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else if (hasImportedData === 'true' && logs.length === 0) {
      DialogService.show({
        name: 'noDataExport',
        type: 'warning',
        title: I18n.translate('scm::ATTENTION'),
        text: I18n.translate('scm::NO_DATA_EXPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else if (hasImportedData === 'false' || hasImportedData === '') {
      DialogService.show({
        name: 'needImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::NEED_IMPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/importacao');
              },
            },
          },
        ],
      });
    }
  }

  public async executeExportacao() {
    await this.joinProdutosSolicitacao();
    await this.joinProdutosInventarios();
    executeExport().then(() => {
      const dateLastSinc = moment().format(this.utilities.getFormatLocaleHour());
      localStorage.setItem('dataUltimaSincronizacao', dateLastSinc);
      this.utilities.lastSynchronization();
      // const errorItem = localStorage.getItem('request-error-log') || '[]';
      // const errors: IDictionary = JSON.parse(errorItem);
      const comp = Metadata.getInstance<Badge>('badgeErrorNotifications');
      this.errorsController.setErrorsBadgeCount({ component: comp });
      if (document.getElementsByName('badgeErrorListItem').length > 0) {
        const comp1 = Metadata.getInstance<Badge>('badgeErrorListItem');
        this.errorsController.setErrorsBadgeCount({ component: comp1 });
      }
      this.clearOldHistoryError();
      const errors = JSON.parse(localStorage.getItem('request-error-log') || '[]');
      if (errors.filter((el: IDictionary) => el.seen === false).length === 0) {
        DialogService.show({
          name: 'successExport',
          type: 'success',
          title: I18n.translate('scm::SUCCESS'),
          text: I18n.translate('scm::SUCCESS_EXPORT'),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                },
              },
            },
          ],
        });
      } else {
        DialogService.show({
          name: 'warningExport',
          type: 'warning',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate('scm::WARNING_EXPORT'),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                },
              },
            },
          ],
        });
      }
    });
  }

  public joinProdutosInventarios() {
    const item = localStorage.getItem('request-log') || '[]';
    const logs = JSON.parse(item);
    [...logs].forEach((log: RequestLog, index: number) => {
      if (log.route === 'produtosInventario' && log.operation === 'create' && log.row.new === true) {
        const indexInventario = logs.findIndex((el: RequestLog) => el.route === 'inventarios'
          && el.operation === 'create' && el.row.CDFILIAL === log.row.CDFILIAL
          && el.row.NRINVENTRETI === log.row.NRINVENTRETI);
        if (indexInventario !== -1) {
          if (!Object.prototype.hasOwnProperty.call(logs[indexInventario].row, 'PRODUTOS')) {
            logs[indexInventario].row.PRODUTOS = [];
          }
          logs[indexInventario].row.PRODUTOS.push(log.row);
          logs.splice(index, 1);
        }
      }
    });
    const stringified = JSON.stringify(logs);
    localStorage.setItem('request-log', stringified);
  }

  public joinProdutosSolicitacao() {
    const filiaisItem = localStorage.getItem('FILIAIS_IMPORTACAO') || '';
    const filiais = filiaisItem.split('_');
    const item = localStorage.getItem('request-log') || '[]';
    const logs = JSON.parse(item);
    filiais.forEach((filial: string) => {
      const produtos = logs.filter((el: RequestLog) => el.route === 'produtosSolicitacao'
        && el.row?.NRSOLICMP === 'A ser criada' && el.row?.CDFILSOLI === filial);
        // && el.row?.IDITMSOLEXTRA === 'N'
      if (produtos.length > 0) {
        const sortedLogs = produtos.sort((a:RequestLog, b:RequestLog) => a.timestamp - b.timestamp);
        const solicitacao: IDictionary = {
          CDFILSOLI: '',
          CDFILLANCA: '',
          DTSOLCMP: '',
          IDTIPLAN: '',
          IDSOLEXTRA: '',
          CDFILCENT: '',
          CDOPERLANC: '',
          DTHRLANCSOL: '',
          QTITEMSOLIC: '',
          // changed: true,
          CDFILICONCATNRSOL: '',
          PRODUTOS: [],
          LOG: [],
        };
        const solicitacaoExtra: IDictionary = {
          CDFILSOLI: '',
          CDFILLANCA: '',
          DTSOLCMP: '',
          IDTIPLAN: '',
          IDSOLEXTRA: '',
          CDFILCENT: '',
          CDOPERLANC: '',
          DTHRLANCSOL: '',
          QTITEMSOLIC: '',
          // changed: true,
          CDFILICONCATNRSOL: '',
          PRODUTOS: [],
          LOG: [],
        };

        // cria propriedade de log
        solicitacao.LOG = sortedLogs.filter((el: RequestLog) => el.row?.IDSOLEXTRA === 'N')
          .map((el: RequestLog) => el.row?.LOG);
        solicitacaoExtra.LOG = sortedLogs.filter((el: RequestLog) => el.row?.IDSOLEXTRA !== 'N')
          .map((el: RequestLog) => el.row?.LOG);

        sortedLogs.forEach((log: RequestLog) => {
          if (log.method === 'post') {
            if (log.row?.IDSOLEXTRA === 'N') {
              solicitacao.PRODUTOS.push(log.row);
              // se for o primeiro produto incluso na solicitação informa os dados da mesma
              if (solicitacao.DTSOLCMP === '') {
                solicitacao.CDFILSOLI = log.row?.CDFILSOLI;
                solicitacao.CDFILLANCA = log.row?.CDFILLANCA;
                solicitacao.DTSOLCMP = log.row?.DTSOLCMP;
                solicitacao.IDTIPLAN = log.row?.IDTIPLAN;
                solicitacao.IDSOLEXTRA = log.row?.IDSOLEXTRA;
                solicitacao.CDFILCENT = log.row?.CDFILCENT;
                solicitacao.CDOPERLANC = log.row?.CDOPERLANC;
                solicitacao.DTHRLANCSOL = log.row?.DTHRLANCSOL;
                solicitacao.QTITEMSOLIC = log.row?.QTITEMSOLIC;
                solicitacao.CDFILICONCATNRSOL = log.row?.CDFILICONCATNRSOL;
              }
            } else {
              solicitacaoExtra.PRODUTOS.push(log.row);
              // se for o primeiro produto incluso na solicitação informa os dados da mesma
              if (solicitacaoExtra.DTSOLCMP === '') {
                solicitacaoExtra.CDFILSOLI = log.row?.CDFILSOLI;
                solicitacaoExtra.CDFILLANCA = log.row?.CDFILLANCA;
                solicitacaoExtra.DTSOLCMP = log.row?.DTSOLCMP;
                solicitacaoExtra.IDTIPLAN = log.row?.IDTIPLAN;
                solicitacaoExtra.IDSOLEXTRA = log.row?.IDSOLEXTRA;
                solicitacaoExtra.CDFILCENT = log.row?.CDFILCENT;
                solicitacaoExtra.CDOPERLANC = log.row?.CDOPERLANC;
                solicitacaoExtra.DTHRLANCSOL = log.row?.DTHRLANCSOL;
                solicitacaoExtra.QTITEMSOLIC = log.row?.QTITEMSOLIC;
                solicitacaoExtra.CDFILICONCATNRSOL = log.row?.CDFILICONCATNRSOL;
              }
            }
          } else if (log.method === 'put') {
            if (log.row?.IDSOLEXTRA === 'N') {
              const index = solicitacao.PRODUTOS.findIndex((el: IDictionary) => el.CDFILSOLI === log.row?.CDFILSOLI
                && el.CDFILLAN === log.row?.CDFILLAN && el.CDPRODUTO === log.row?.CDPRODUTO
                && el.DTPROGENTR === log.row?.DTPROGENTR);
              if (index !== -1) {
                // solicitacao.PRODUTOS[index] = log.row;
                solicitacao.PRODUTOS[index].QTAJUSTADA = log.row?.QTAJUSTADA;
                solicitacao.PRODUTOS[index].DSJUSTOPER = log.row?.DSJUSTOPER;
              }
            } else {
              const index = solicitacaoExtra.PRODUTOS.findIndex((el: IDictionary) => el.CDFILSOLI === log.row?.CDFILSOLI
                && el.CDFILLAN === log.row?.CDFILLAN && el.CDPRODUTO === log.row?.CDPRODUTO
                && el.DTPROGENTR === log.row?.DTPROGENTR);
              if (index !== -1) {
                // solicitacaoExtra.PRODUTOS[index] = log.row;
                solicitacaoExtra.PRODUTOS[index].QTAJUSTADA = log.row?.QTAJUSTADA;
                solicitacaoExtra.PRODUTOS[index].DSJUSTOPER = log.row?.DSJUSTOPER;
              }
            }
          } else if (log.method === 'delete') {
            if (log.row?.IDSOLEXTRA === 'N') {
              const index = solicitacao.PRODUTOS.findIndex((el: IDictionary) => el.CDFILSOLI === log.row?.CDFILSOLI
                && el.CDFILLAN === log.row?.CDFILLAN && el.CDPRODUTO === log.row?.CDPRODUTO
                && el.DTPROGENTR === log.row?.DTPROGENTR);
              if (index !== -1) {
                solicitacao.PRODUTOS.splice(index, 1);
              }
            } else {
              const index = solicitacaoExtra.PRODUTOS.findIndex((el: IDictionary) => el.CDFILSOLI === log.row?.CDFILSOLI
                && el.CDFILLAN === log.row?.CDFILLAN && el.CDPRODUTO === log.row?.CDPRODUTO
                && el.DTPROGENTR === log.row?.DTPROGENTR);
              if (index !== -1) {
                solicitacaoExtra.PRODUTOS.splice(index, 1);
              }
            }
          }
          const indexLog = logs.map((el: RequestLog) => el.timestamp).indexOf(log.timestamp);
          if (indexLog !== -1) {
            logs.splice(indexLog, 1);
          }
        });

        if (solicitacao.PRODUTOS.length > 0) {
          solicitacao.QTITEMSOLIC = solicitacao.PRODUTOS.length;
          const newLog : RequestLog = {
            method: 'post',
            url: 'solicitacao',
            row: solicitacao,
            operation: 'create',
            timestamp: Date.now(),
            route: 'solicitacao',
          };
          // adiciona a solicitacao nos logs para enviar na exportacao
          logs.push(newLog);
        }

        if (solicitacaoExtra.PRODUTOS.length > 0) {
          solicitacaoExtra.QTITEMSOLIC = solicitacaoExtra.PRODUTOS.length;
          const newLogExtra : RequestLog = {
            method: 'post',
            url: 'solicitacao',
            row: solicitacaoExtra,
            operation: 'create',
            timestamp: Date.now(),
            route: 'solicitacao',
          };
          // adiciona a solicitacao nos logs para enviar na exportacao
          logs.push(newLogExtra);
        }
      }
    });
    // sobrescreve logs, para tirar
    const stringified = JSON.stringify(logs);
    localStorage.setItem('request-log', stringified);
  }

  public clearOldHistoryError() {
    const errors = JSON.parse(localStorage.getItem('request-error-log') || '[]');
    const errorsFiltered = errors.filter((el: IDictionary) => !moment(el.timestamp).isBefore(moment().subtract(7, 'days')));
    const stringified = JSON.stringify(errorsFiltered);
    localStorage.setItem('request-error-log', stringified);
  }
}
